package com.caimei.service.impl;

import com.caimei.mapper.AuthProductMapper;
import com.caimei.model.ResponseJson;
import com.caimei.model.dto.ProductSaveDto;
import com.caimei.model.po.ProductParamPo;
import com.caimei.model.po.ProductPo;
import com.caimei.model.vo.ProductFormVo;
import com.caimei.model.vo.ProductListVo;
import com.caimei.service.AuthProductService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

/**
 * Description
 *
 * @author : Aslee
 * @date : 2021/5/17
 */
@Slf4j
@Service
public class AuthProductServiceImpl implements AuthProductService {

    @Resource
    private AuthProductMapper authProductMapper;

    @Override
    public ResponseJson<PageInfo<ProductListVo>> getProductList(Integer authId, String productName, String snCode, Integer pageNum, Integer pageSize) {
        if (null == authId) {
            return ResponseJson.error("参数异常，请输入授权id", null);
        }
        PageHelper.startPage(pageNum, pageSize);
        List<ProductListVo> productList = authProductMapper.getProductList(authId, productName, snCode);
        PageInfo<ProductListVo> pageData = new PageInfo<>(productList);
        return ResponseJson.success(pageData);
    }

    @Override
    public ResponseJson updateProductStatus(Integer productId, Integer status) {
        if (productId == null) {
            return ResponseJson.error("请输入用户id");
        }
        if (status == null) {
            return ResponseJson.error("请输入要更新的状态值");
        } else if (status != 0 && status != 1) {
            return ResponseJson.error("状态值只能为0或1");
        }
        authProductMapper.updateProductStatusByProductId(productId, status);
        if (status == 0) {
            return ResponseJson.success("下架授权商品成功");
        } else {
            return ResponseJson.success("上架授权商品成功");
        }
    }

    @Override
    public ResponseJson deleteProduct(Integer productId) {
        if (null == productId) {
            return ResponseJson.error("参数异常，请输入授权商品id");
        }
        // 删除商品
        authProductMapper.deleteProductByProductId(productId);
        // 删除商品参数
        authProductMapper.deleteParamsByProductId(productId);
        return ResponseJson.success("删除授权商品成功");
    }

    @Override
    public ResponseJson saveProduct(ProductSaveDto productSaveDto) {
        Integer productId = productSaveDto.getProductId();
        Integer authId = productSaveDto.getAuthId();
        String productName = productSaveDto.getProductName();
        String snCode = productSaveDto.getSnCode();
        String productImage = productSaveDto.getProductImage();
        String certificateImage = productSaveDto.getCertificateImage();
        Integer status = productSaveDto.getStatus();
        Integer createBy = productSaveDto.getCreateBy();
        List<ProductParamPo> paramList = productSaveDto.getParamList();
        if (null == authId) {
            return ResponseJson.error("参数异常，请输入授权id", null);
        }
        if (StringUtils.isBlank(productName)) {
            return ResponseJson.error("参数异常，请输入商品名称", null);
        }
        if (StringUtils.isBlank(snCode)) {
            return ResponseJson.error("参数异常，请输入商品SN码", null);
        }
        Integer productIdBySnCode = authProductMapper.getProductIdBySnCode(snCode);
        if (null != productIdBySnCode && !productIdBySnCode.equals(productId)) {
            return ResponseJson.error("参数异常，该商品SN码已存在，请重新输入", null);
        }
        if (StringUtils.isBlank(productImage)) {
            return ResponseJson.error("参数异常，请上传商品图片", null);
        }
        if (StringUtils.isBlank(certificateImage)) {
            return ResponseJson.error("参数异常，请上传授权牌照", null);
        }
        if (null == status) {
            return ResponseJson.error("参数异常，请输入商品状态值", null);
        } else if (status != 0 && status != 1) {
            return ResponseJson.error("商品状态值只能为0或1");
        }
        if (null == paramList || paramList.size() <= 0) {
            return ResponseJson.error("参数异常，商品参数列表不能为空", null);
        }
        // 是否为添加操作
        boolean insertFlag = null == productId;
        /*
            组装商品数据
         */
        ProductPo product = new ProductPo();
        // 授权id
        product.setAuthId(authId);
        // 商品名称
        product.setProductName(productName);
        // sn码
        product.setSnCode(snCode);
        // 商品图片
        product.setProductImage(productImage);
        // 授权牌照
        product.setCertificateImage(certificateImage);
        // 上架状态：0下架，1上架
        product.setStatus(status);
        if (insertFlag) {
            // 创建人id
            product.setCreateBy(createBy);
            // 创建时间
            product.setCreateTime(new Date());
            // 插入授权商品
            authProductMapper.insertProduct(product);
        } else {
            // 商品id
            product.setProductId(productId);
            // 更新授权商品
            authProductMapper.updateProductByProductId(product);
        }
        // 删除商品参数
        authProductMapper.deleteParamsByProductId(product.getProductId());
        // 保存商品参数
        paramList.forEach(param->{
            if (StringUtils.isNotBlank(param.getParamName()) && StringUtils.isNotBlank(param.getParamContent())) {
                authProductMapper.insertProductParam(product.getProductId(), param.getParamName(), param.getParamContent());
            }
        });
        return ResponseJson.success("保存授权商品成功");
    }

    @Override
    public ResponseJson<ProductFormVo> getProductFormData(Integer productId) {
        if (null == productId) {
            return ResponseJson.error("参数异常，请输入商品id", null);
        }
        ProductFormVo productForm = authProductMapper.getProductFormByProductId(productId);
        if (null == productForm) {
            return ResponseJson.error("商品不存在", null);
        }
        List<ProductParamPo> paramList = authProductMapper.getParamsByProductId(productId);
        productForm.setParamList(paramList);
        return ResponseJson.success(productForm);
    }

    @Override
    public List<Integer> getProductIdsByAuthId(Integer authId) {
        return authProductMapper.getProductIdsByAuthId(authId);
    }
}
